package ir_graph;

import syntax.expressions.*;
import syntax.headers.*;
import java.util.*;
import parsing.*;


public class Block
{
	Block parent;
	Hashtable<String, DefBody> definitions;
	
	
	
	public class DefBody
	{
		public Symbol symbol;
		public RightHandSide rhs;
		public Block body;
	}
	
	public void add(Expression guard, Definition def, List<Tree<Header>> children)
	{
		Symbol sym = def.symbol;
		if (definitions.containsKey(sym.name))
		{
			throw new SymbolException(
					"Symbol " + sym.name + " defined at " + 
					sym.start.toString() + " and " + 
					definitions.get(sym.name).symbol.start.toString());
		}
		else
		{
			DefBody defbody = new DefBody();
			defbody.symbol = def.symbol;
			defbody.rhs = def.rhs;
			
			defbody.body = makeFromParseTrees(children);
			definitions.put(sym.name, defbody);
		}
	}
	
	
	public static Block makeFromParseTrees(List<Tree<Header>> headers)
	{
		Block block = new Block();
		
		block.addDefinitions(null, headers);
		
		return block;
	}
	
	public void addDefinitions(Expression guard, List<Tree<Header>> headers)
	{
		
		
		for (Tree<Header> tree : headers) {
			Header h = tree.content;
			
			if (h instanceof Definition)
			{
				add(guard, (Definition) h, tree.children);
				continue;
			}
			if (h instanceof NamedFunctionDef)
			{
				NamedFunctionDef fun = (NamedFunctionDef) h;
				
				AnonFunctionDef anon = new AnonFunctionDef();
				anon.params = fun.params;
				anon.result = fun.result;
				anon.start  = fun.start;
				anon.end    = fun.end;
				
				Definition def = new Definition(fun.name, anon);
				def.start = fun.start;
				
				add(guard, def, tree.children);
				
				continue;
			}
			
		}
		
		
		
	}
	

	public void eval(String symbolName)
	{

	}
}

